import logging
import re

from django.conf import settings
from django.contrib import admin
from django.urls import reverse
from django.utils.deprecation import MiddlewareMixin

logger = logging.getLogger(__name__)

def check_url(base, url):
    " 判断当前选中菜单 "

    if base == '/admin/':
        return base == url
    else:
        return base in url

def get_normal_app_model(app_list, index, current):
    " 根据applabel获取label和链接 "
    children = []

    for item in app_list[index]['models']:
        children.append({
            'label': item['name'],
            'path': item['admin_url'],
            'active': check_url(item['admin_url'], current)
            })

    return children

def get_app_model(app_list, name, current):
    " 根据app.model 获取对应的label和链接 "
    app, model_name = name.split('.')

    for item in app_list:
        if app.upper() == item['app_label'].upper():
            for model in item['models']:

                if model['object_name'].upper() == model_name.upper():
                    label = item['label'] if 'label' in item.keys() else model['name']
                    path = model['admin_url']

                    return {'label':label,
                            'path':path,
                            'active':check_url(path, current)}

class LeftMenuMiddleware(MiddlewareMixin):


    def process_template_response(self, request, response):
        login_bg = getattr(settings, 'JUSS_LOGIN_BG', None)
        if getattr(response, 'context_data'):
            response.context_data['login_bg'] = login_bg

        if not request.user.is_authenticated or not re.match(r'\/admin\/*', request.path_info):
            return response

        if not 'available_apps' in response.context_data:
            return response

        app_list = response.context_data['available_apps']
        new_menu = []
        current = request.path_info
        menu = getattr(settings, 'JUSS_LEFT_MENU', False)

        if menu:
            for i, item in enumerate(menu):
                new_menu.append({
                    'label':item['label'],
                    'children': []
                    })

                for link in item['children']:

                    if 'model' in link:
                        model = get_app_model(app_list, link['model'], current)

                        if model:
                            new_menu[-1]['children'].append({
                                'label': link['label'] if 'label' in link.keys() else model['label'],
                                'path': model['path'],
                                'active': model['active']
                                })
                    else:
                        new_menu[-1]['children'].append({
                            'label': link['label'],
                            'path': link['path'],
                            'active': check_url(link['path'], current)
                        })

                if not new_menu[-1]['children']:
                    del new_menu[-1]

        else:

            for i, item in enumerate(app_list):
                new_menu.append({
                    'label': item['name'],
                    'children': get_normal_app_model(app_list, i, current),
                    })

        response.context_data['juss_left_menu'] = new_menu

        return response



from django.contrib.postgres.forms import SimpleArrayField, JSONField
from .widgets import *

class MultipleImageField(SimpleArrayField):
    widget = MultipleImageWidget


class JSONEditField(JSONField):
    widget = JSONWidget



import logging
from django.contrib.postgres.fields import ArrayField, JSONField
from django.db import models
from . import forms
from . import widgets

logger = logging.getLogger(__name__)


class MultipleImageField(ArrayField):
    def __init__(self, base_field=None, size=2, **kwargs):
        super().__init__(models.URLField(), size, **kwargs)

    def formfield(self, **kwargs):
        return super().formfield(**{
            'form_class': forms.MultipleImageField,
            'base_field': self.base_field.formfield(),
            'max_length': self.size,
            **kwargs,
        })


class JSONEditField(JSONField):
    def __init__(self, verbose_name=None, name=None, encoder=None, template={}, **kwargs):
        if encoder and not callable(encoder):
            raise ValueError("The encoder parameter must be a callable object.")
        self.encoder = encoder
        self.template = template
        super().__init__(verbose_name, name, **kwargs)


    def formfield(self, **kwargs):
        res = super().formfield(**{
            'form_class': forms.JSONEditField,
            **kwargs
        })
        res.widget.attrs.update({'template':self.template})
        return res

    def widget_attrs(self, widget):
        attrs = super().widget_attrs(widget)
        attrs.setdefault('template', 'teststst')
        return attrs



import logging

from django.conf import settings
from django.core.files.storage import default_storage
from django.forms.widgets import ClearableFileInput, Select, Textarea, URLInput, TextInput, Textarea

logger = logging.getLogger(__name__)

class HtmlEditor(Textarea):
    def __init__(self, *args, **kwargs):
        super(HtmlEditor, self).__init__(*args, **kwargs)
        self.attrs['class'] = 'html-editor'

    class Media:
        css = {
            'all': (
                'bower_components/codemirror/lib/codemirror.css',
            )
        }
        js = (
            'bower_components/codemirror/lib/codemirror.js',
            'bower_components/codemirror/mode/xml/xml.js',
            'bower_components/codemirror/mode/htmlmixed/htmlmixed.js',
            'juss/widgets/htmleditor.js'
        )

class RichTextWidget(Textarea):
    class Media:
        js = ('bower_components/tinymce/tinymce.min.js',
                'juss/widgets/richtext.js')

class JFileInputWidget(ClearableFileInput):
    template_name = 'juss/widgets/jfileinput.html'

class JImageWidget(URLInput):
    " 单图 "
    template_name = 'juss/widgets/jimage.html'

class MultipleImageWidget(TextInput):
    " 多图 "
    template_name = 'juss/widgets/multiple_image.html'


class JSONWidget(Textarea):
    " JSON "
    template_name = 'juss/widgets/json.html'

class JMSelectWidget(Select):
    template_name = 'juss/widgets/jmselect.html'

    class Media:
        css = {
            'all': ('juss/widgets/micromodal.css',)
        }
        js = ('https://unpkg.com/micromodal/dist/micromodal.min.js', 'juss/widgets/jfileselect.js')

    def create_option(self, name, value, label, selected, index, subindex=None, attrs=None):
        data = super().create_option(name, value, label, selected, index, subindex=None, attrs=None)
        data['url'] = default_storage.url(value.replace(settings.MEDIA_ROOT, ''))

        return data

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['widget']['optgroups'] = self.optgroups(name, context['widget']['value'], attrs)

        value = context['widget']['value'][0]

        if value:
            context['widget']['url'] = default_storage.url(value.replace(settings.MEDIA_ROOT, ''))

        return context
